# coding: UTF-8
import serial
import time

class RCB4_com():
    def __init__(self):
        self.debug_flg = False #デバック用フラグ(Trueなら攻撃しない)
        self.p_acceleration = 393   #起立時の加速度センサーの値
        self.p_edge = 53            #崖検出距離
        self.range_border = 20.0    #PSD切り替え距離(短距離用/中距離用)

    def com_open(self,num): #シリアル通信オープン
        try:
            self.ser = serial.Serial(num-1, 115200,parity=serial.PARITY_EVEN,timeout=1)
            return(0)
        except:
            return(-1)
    def com_close(self): #シリアル通信クローズ
        try:
            self.ser.close()
            return(0)
        except:
            return(-1)

    def go_forward(self,num): #前進
        num = int(num)
        print(u"前進"+str(num)+u"歩")

        for i in range(num//2):
            self.search_psd(0,0)
            if self.edge_flg == 1:
                print(u"崖検出・前進中止")
                break
            self.com_line = [0x0D,0x00,0x02,0x50,0x03,0x00,0x02,0x01,0x00,0x00,0x00,0x00,0x65]
            self.start_motion()
        if num%2 == 1:
            self.search_psd(0,0)
            if self.edge_flg == 1:
                print(u"崖検出・前進中止")
            else:
                self.com_line = [0x0D,0x00,0x02,0x50,0x03,0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x63]
                self.start_motion()

    def go_back(self,num): #後進
        num = int(num)
        print(u"後進"+str(num)+u"歩")

        for i in range(num//2):
            self.com_line = [0x0D,0x00,0x02,0x50,0x03,0x00,0x02,0x02,0x00,0x00,0x00,0x00,0x66]
            self.start_motion()
        if num%2 == 1:
            self.com_line = [0x0D,0x00,0x02,0x50,0x03,0x00,0x00,0x02,0x00,0x00,0x00,0x00,0x64]
            self.start_motion()
    def go_left(self,num): #左移動
        print(u"左移動"+str(num)+u"歩")
        for i in range(num):
            self.com_line = [0x0D,0x00,0x02,0x50,0x03,0x00,0x00,0x08,0x00,0x00,0x00,0x00,0x6A]
            self.start_motion()
    def go_right(self,num): #右移動
        print(u"右移動"+str(num)+u"歩")
        for i in range(num):
            self.com_line = [0x0D,0x00,0x02,0x50,0x03,0x00,0x00,0x04,0x00,0x00,0x00,0x00,0x66]
            self.start_motion()
    def rotate_left(self,ang): #左回転
        print(u"左回転"+str(ang)+u"度")
        for i in range(ang//30):
            self.com_line = [0x0D,0x00,0x02,0x50,0x03,0x00,0x00,0x0A,0x00,0x00,0x00,0x00,0x6C]
            self.start_motion()
        for i in range((ang%30)//10):
            self.com_line = [0x0D,0x00,0x02,0x50,0x03,0x00,0x02,0x0A,0x00,0x00,0x00,0x00,0x6E]
            self.start_motion()
    def rotate_right(self,ang): #右回転
        print(u"右回転"+str(ang)+u"度")
        for i in range(ang//30):
            self.com_line = [0x0D,0x00,0x02,0x50,0x03,0x00,0x00,0x06,0x00,0x00,0x00,0x00,0x68]
            self.start_motion()
        for i in range((ang%30)//10):
            self.com_line = [0x0D,0x00,0x02,0x50,0x03,0x00,0x02,0x06,0x00,0x00,0x00,0x00,0x6A]
            self.start_motion()

    def chk_tumble(self): #転倒検出
        print(u"転倒チェック")
        self.com_line = [0x0A,0x00,0x20,0x00,0x00,0x00,0x28,0x00,0x02,0x54] #AD3を読む
        self.ser.write(self.com_line)
        self.receive_data()
        vout = self.s2i(self.r_data[2:4])

        if vout > self.p_acceleration+25:
            print(u"仰向け状態")
            return(1)
        elif vout < self.p_acceleration-25:
            print(u"うつぶせ状態")
            return(1)
        else:
            print(u"起立状態")
            return(0)
    def stand_up(self): #起き上がり
        time.sleep(1)

        self.com_line = [0x0A,0x00,0x20,0x00,0x00,0x00,0x28,0x00,0x02,0x54] #AD3を読む
        self.ser.write(self.com_line)
        self.receive_data()
        vout = self.s2i(self.r_data[2:4])

        if vout > self.p_acceleration+25:
            print(u"起き上がり(仰向け)")
            self.com_line = [0x0D,0x00,0x02,0x50,0x03,0x00,0x10,0x01,0x00,0x00,0x00,0x00,0x73]
            self.start_motion()
        elif vout < self.p_acceleration-25:
            print(u"起き上がり(うつぶせ)")
            self.com_line = [0x0D,0x00,0x02,0x50,0x03,0x00,0x10,0x02,0x00,0x00,0x00,0x00,0x74]
            self.start_motion()

    def attack(self,num): #攻撃
        if num == 0:
            print(u"前転キック")
            if self.debug_flg == False:
                self.rotate_left(30)
                self.com_line = [0x0D,0x00,0x02,0x50,0x03,0x00,0x04,0x10,0x00,0x00,0x00,0x00,0x76]
                self.start_motion()
        elif num == 1:
            print(u"投げ")
            if self.debug_flg == False:
                self.com_line = [0x0D,0x00,0x02,0x50,0x03,0x00,0x02,0x10,0x00,0x00,0x00,0x00,0x74]
                self.start_motion()
        elif num == 2:
            print(u"ダブルパンチ")
            if self.debug_flg == False:
                self.com_line = [0x0D,0x00,0x02,0x50,0x03,0x00,0x00,0x10,0x00,0x00,0x00,0x00,0x72]
                self.start_motion()

    def move_waist(self,ang,sp): #腰を回転
        pos = int(ang*4000/135+7500)
        self.com_line = [0x07,0x0F,0x01]
        self.com_line.append(sp)
        self.com_line += self.i2s(pos)
        self.add_CSUM()
        self.ser.write(self.com_line)
        self.receive_data()

    def search_psd(self,ang1,ang2): #PSDで物体検出
        self.move_waist(ang1,abs(ang1))
        time.sleep(abs(ang1)*0.02)

        ang = ang1
        self.ang_list = []
        self.dist_list = []
        self.min_dist = 10000.0

        self.edge_list = []
        self.edge_flg = 0

        while True:
            #距離計算
            psds = self.get_psd_s()
            psdm = self.get_psd_m()

            if psds < self.range_border: #これ以下は短距離用、それ以遠は中距離用を採用
                dist = psds
            else:
                dist = psdm
                if dist < self.range_border:
                    dist = self.range_border #ボーダー以下の値を出さないよう調整
            self.ang_list.append(ang)
            self.dist_list.append(dist)

            if dist < self.min_dist:
                self.min_dist = dist
                self.detect_ang = ang

            #崖判定
            psde = self.get_psd_edge()
            if psde > self.p_edge:   #これより大きければ崖と判定
                self.edge_flg = 1
                self.edge_list.append(1) #崖なら1
            else:
                self.edge_list.append(0) #床なら0

            ang += 10
            if ang>ang2:
                break

            self.move_waist(ang,10)
            time.sleep(0.2)

        self.move_waist(0,abs(ang2))
        time.sleep(abs(ang2)*0.02)

        time.sleep(0.5) #念のため0.5秒待つ

    def chk_bat(self): #バッテリ電圧チェック(AD0)
        self.com_line = [0x0A,0x00,0x20,0x00,0x00,0x00,0x22,0x00,0x02,0x4E]
        self.ser.write(self.com_line)
        self.receive_data()
        vout = self.s2i(self.r_data[2:4])
        v_bat = vout*25.0/1024.0
        return(v_bat)

    def get_psd_s(self): #PSDセンサー短距離用(AD5)
        self.com_line = [0x0A,0x00,0x20,0x00,0x00,0x00,0x2C,0x00,0x02,0x58]
        self.ser.write(self.com_line)
        self.receive_data()
        vout = self.s2i(self.r_data[2:4])
        dist = 16067*(vout**-1.202)
        return(dist)
    def get_psd_m(self): #PSDセンサー中距離用(AD6)
        self.com_line = [0x0A,0x00,0x20,0x00,0x00,0x00,0x2E,0x00,0x02,0x5A]
        self.ser.write(self.com_line)
        self.receive_data()
        vout = self.s2i(self.r_data[2:4])
        dist = -2.77754410476235E-11*vout**5 + 5.35650049439969E-08*vout**4 - 4.14005320508988E-05*vout**3 + 1.62236534142710E-02*vout**2 - 3.35987865787528E+00*vout + 3.41755390174423E+02
        return(dist)
    def get_psd_edge(self): #PSDセンサー崖検出用(AD10)
        self.com_line = [0x0A,0x00,0x20,0x00,0x00,0x00,0x36,0x00,0x02,0x62]
        self.ser.write(self.com_line)
        self.receive_data()
        vout = self.s2i(self.r_data[2:4])
        dist = 16067*(vout**-1.202)
        return(dist)

    #以下は内部使用の関数
    def start_motion(self):
        self.ser.write(self.com_line) #コマンド送信
        self.receive_data()

        self.com_line = [0x0D,0x00,0x02,0x50,0x03,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x62] #ニュートラル送信
        self.ser.write(self.com_line)
        self.receive_data()

        while True:
            time.sleep(0.2) #0.2秒ごとに終了をチェック
            self.com_line = [0x0A,0x00,0x20,0x00,0x00,0x00,0x00,0x00,0x01,0x2B]
            self.ser.write(self.com_line)
            self.receive_data()
            if self.r_data[2]&0b00001000 != 0:
                break

        time.sleep(0.5) #念のため0.5秒待つ
        print(u"完了")

    def receive_data(self):
        try:
            self.r_data = [ord(self.ser.read())]
            for i in range(self.r_data[0]-1):
                self.r_data.append(ord(self.ser.read()))
        except:
            print(u"通信受信失敗")

    def s2i(self,data): #Short(2バイト)をIntに変換
        num = data[1]*256+data[0]
        if num >= 0x8000:
            num -= 0x10000
        return(num)

    def i2s(self,num): #IntをShort(2バイト)に変換
        if num < 0:
            num += 0x10000

        data = []
        data.append(num%256)
        data.append(num//256)

        return(data)

    def add_CSUM(self): #CSUMを計算してコマンド配列の最後に追加
        csum = sum(self.com_line)
        csum = csum%256
        self.com_line.append(csum)
